/*        Demo of Arduino LCD/Keypad Shield
 
 Demonstrates how to scan the keypad and output to the LCD display
 on the ubiquitous Arduino LCD/keypad shield.

 The LiquidCrystal library works with all LCD displays that are
 compatible with the Hitachi HD44780 driver. There are many of
 them out there, usually with the same 16-pin interface.  The
 Arduino LCD/keypad shield typically uses the TC1602A LCD assembly.
 
 Scanning the keypad is so simple, no library is required.
 
  by R.Waters, 16-Oct-2012
 
*****************************************************************/
//
// Tell the compiler where to find the functions defined in the
// LiquidCrystal library so it won't complain when we use them.
//
#include <LiquidCrystal.h> // Library to drive the TR1602 LCD display
//_______________________________________________________________
//
//         **** SPECIAL NOTE on ENUM data types ****
//
// An enumerated data type is a collection of related symbolic values
// that make your code easier to read and understand.  For example,
// the data type BEER could include values ALE, PILSNER, and STOUT.
// This is called an "enumerated" type because the compiler treats
// all these values as numbers (integers).  However, due to the way
// the compiler operates, full functionality of a custom enumerated
// type is not available when the type is defined in a single-file
// sketch like this one.  The "correct" way to fix this is to place
// the type definition in a separate header file (named *.h) so the
// compiler can read it early during the compilation process. Since
// this sketch is in a single file for simplicity, we cannot use an
// enumerated type returned as such from a function.  Attempting it
// will create a compilation error.  We will instead use an integer
// where needed and cast (convert) it into the enum type as needed.
// This kludge is only needed to keep this sketch in a single file.
//
// Define the keypad codes for the LCD/Keypad shield.
enum  KEY_ID
{
  KEY_NONE = 0, // 0 = no key depressed
  KEY_RIGHT,    // 1
  KEY_UP,       // 2
  KEY_DOWN,     // 3
  KEY_LEFT,     // 4
  KEY_SELECT    // 5
};
//_______________________________________________________________
//
//  DEFINE our usage of the 14 DIGITAL I/O pins
//  as hard-wired by the LCD/keypad shield.
//
//        PIN_XXX = 0;  // digital pin  0 is used by USB for Rx data
//        PIN_XXX = 1;  // digital pin  1 is used by USB for Tx data
//        PIN_XXX = 2;  // digital pin  2 is available
//        PIN_XXX = 3;  // digital pin  3 is available
const int PIN_DB4 = 4;  // digital pin  4 drives LCD bit4
const int PIN_DB5 = 5;  // digital pin  5 drives LCD bit5
const int PIN_DB6 = 6;  // digital pin  6 drives LCD bit6
const int PIN_DB7 = 7;  // digital pin  7 drives LCD bit7
const int PIN_RS  = 8;  // digital pin  8 drives LCD Register Select (cmd/data)
const int PIN_WRT = 9;  // digital pin  9 drives LCD Write Enable (data strobe)
const int PIN_LED = 10; // digital pin 10 drives LCD backlight in PWM mode
//        PIN_XXX = 11; // digital pin 11 is available
//        PIN_XXX = 12; // digital pin 12 is available
const int PIN_MON = 13; // digital pin 13 runs the on-board LED
//
// Define the usage of the 6 ANALOG INPUT pins
//
const int  ADC_KEYPAD  = 0; // ADC ch 0 reads LCD shield's 5-key pad
//         ADC_XXXXX   = 1; // ADC ch 1 not assigned
//         ADC_XXXXX   = 2; // ADC ch 2 not assigned
//         ADC_XXXXX   = 3; // ADC ch 3 not assigned
//         ADC_XXXXX   = 4; // ADC ch 4 not assigned
//         ADC_XXXXX   = 5; // ADC ch 5 not assigned
//______________________________________________________________
//
// The term "instantiate" means to create an "instance" of an object.
// Essentially, this loads the library code into memory and performs
// whatever initialization is required to prepare it for use.
//
// Instantiate the LiquidCrystal library & assign the interface pins.
// The LCD's R/W* input is hard-wired to a logic LOW which means that
// the CPU can only Write to the LCD, and cannot read back its status.
//
LiquidCrystal lcd(PIN_RS,  // LCD Register Select (cmd vs data)
                  PIN_WRT, // LCD Write Enable (data strobe)
                  PIN_DB4, PIN_DB5, PIN_DB6, PIN_DB7); // LCD data bits 4-7
//______________________________________________________________
//
// GLOBAL Variables are defined and optionally initialized here.
//
int           ledBrt = 120; // LCD backlight range is 0 to 255
KEY_ID        key[3];       // GetKeyCmd() recent decoded key commands
//______________________________________________________________
//
//               @
//               @
//   @@@@  @@@  @@@ @   @ @@@@
//  @     @   @  @  @   @ @   @
//   @@@  @@@@@  @  @   @ @   @
//      @ @      @  @   @ @   @
//  @@@@   @@@@  @@  @@@@ @@@@
//                        @
//                        @
//
// The setup() function is called ONCE by the Arduino framework
// to perform nonrecurring initialization of the system.
//
void setup()
{
  // Initialize the LCD backlight LED
  pinMode(     PIN_LED, OUTPUT); // LCD backlight is driven from pin 10
  analogWrite( PIN_LED, ledBrt); // set the brightness of the LED via PWM
  
  pinMode(     PIN_MON, OUTPUT); // LED flashes when a key is pressed
  digitalWrite(PIN_MON, LOW);    // Start with LED OFF

  // Finish configuration of the LiquidCrystal object.
  // Its interface pins were assigned during instantiation, so
  // the remaining configuration for 1602 display is done here:
  lcd.begin(16, 2);  // Configure LCD driver for 2 rows of 16 chars each
  lcd.setCursor(0, 1);   // lower line
  lcd.print("Press a key");

  int n=sizeof(key)/sizeof(key[0]); // # elements in key[] array
  for (int i=0; i<n; i++)  // fill keystroke array with KEY_NONE
       key[i] = KEY_NONE;
}

//______________________________________________________________
//
//  @@
//   @
//   @    @@@   @@@  @@@@
//   @   @   @ @   @ @   @
//   @   @   @ @   @ @   @
//   @   @   @ @   @ @   @
//  @@@   @@@   @@@  @@@@
//                   @
//                   @
void loop()
{
  // Read the keypad and display its voltage on LCD top row
  unsigned int uCounts = uReadKeypadCounts();
  
  // Convert keypad counts to cmd & display on LCD bottom row
  KEY_ID key = (KEY_ID)GetKeyCmd(uCounts);

  // The C "trinary" operator is a handy way to assign
  // one of two values based on the result of a test.  
  byte state = (key == KEY_NONE) ? LOW : HIGH;
  digitalWrite(PIN_MON, state); // Blink LED for each cmd
  
  delay(50);  // Poll keypad at 20 Hz
}

//___________________________________________________________________
//
// Read & display the keypad command voltage on the LCD upper line.
//
// Returns:  Keypad voltage in ADC counts (0 to 1023)
//
unsigned int uReadKeypadCounts()
{
  // Let the compiler do some of the math to reduce CPU loading.
  // Prepare the A/D converter's conversion factor.
  const  float  VOLTSPERCOUNT = 5.0f / 1024;
  
  // Read the keypad command voltage on ADC channel 0
  unsigned int  adcCounts = analogRead(ADC_KEYPAD);
  
  // Convert raw ADC counts into engineering units
  // Min reading (0 counts)    = Ground potential      (0.000 V)
  // Max Reading (1023 counts) = 1 LSB less than AVref (4.995 V)
  float         rVDC      = (float)adcCounts * VOLTSPERCOUNT;

  // Display the voltage on the upper line of the LCD
  lcd.setCursor(0, 0); // column, line
  lcd.print("ADC0 = ");
  lcd.print(rVDC, 3);  // 2nd arg = # digits past decimal
  lcd.print(" VDC");

  return adcCounts;  // return raw ADC counts to caller
}

//___________________________________________________________________
//
// Convert raw ADC counts into enumerated keypad command as a KEY_ID.
// See the "SPECIAL NOTE on ENUM data types" above for an explanation
// of why we actually return an integer instead of a KEY_ID.
//
int /*KEY_ID*/ GetKeyCmd(unsigned int adcCounts)
{
  KEY_ID code = (KEY_ID)ConvertKey(adcCounts); // get keystroke cmd

  key[2] = key[1];  // Ripple the last 3 key scans thru our buffer
  key[1] = key[0];
  key[0] = code;

  // Detect a keystroke based on specific criteria
  if ((key[2] == key[1]) || // if no change has recently occurred, or
      (key[1] != key[0]))   // the last command is not stable, then
       return (KEY_NONE);   // ignore the value.

  char* pMsg;   // This varb receives a msg for the key cmd
  switch (code) // a "switch" works like a series of "if" statements
  {
    case KEY_RIGHT:   pMsg = "Right";  break;
    case KEY_UP:      pMsg = "Up";     break;
    case KEY_DOWN:    pMsg = "Down";   break;
    case KEY_LEFT:    pMsg = "Left";   break;
    case KEY_SELECT:  pMsg = "Select"; break;
    default:          pMsg = "<none>";
  }
  lcd.setCursor(0, 1);   // lower line
  lcd.print("Command: ");
  lcd.print(pMsg);
  lcd.print("     ");
  return code;
}

//_________________________________________________________________
//
//                                                      K E Y P A D
//
// The keypad on the LCD shield contains a resistor/switch network
// much like the cruise control buttons on a car steering wheel.
// Each switch creates a unique voltage divider ratio, powered by
// the +5V regulator and read by the Channel 0 A/D converter.
//
//                +5V
//                 |
//               2000 ohms
//                 |
//   +----RIGHT----+---------> ADC0 Input
//   |             |
//   |            330 ohms
//   |             |
//   +-----UP------+
//   |             |
//   |            620 ohms
//   |             |
//   +----DOWN-----+
//   |             |
//   |           1000 ohms
//   |             |
//   +----LEFT-----+
//   |             |
//   |           3300 ohms
//   |             |
//   +---SELECT----+
//   |
//  GND
//
//  The 10-bit ADC reads from 0 Volts (0 counts) to +5 Volts
// (1023 counts), depending on which keyswitch is depressed.
// "Ideal" design voltages are listed in this table:
//
//           ---------------DESIGN------------
//  KEY        Resistor Ratio     VDC   counts
//  ------   -----------------   -----  ------
//  RIGHT       0/2000 = 0.000   +0.00      0
//  UP        330/2330 = 0.142   +0.71    145
//  DOWN      950/2950 = 0.322   +1.61    329
//  LEFT     1950/3950 = 0.494   +2.47    505
//  SELECT   5250/7250 = 0.724   +3.62    741
//  (none)   2000/2000 = 1.000   +5.00   1023
//
// Map the ADC reading into a KEY_ID
//
int /*KEY_ID*/ ConvertKey(unsigned int adcCounts)
{
  // Inline code is simple & fast; test most likely case first.
  // Comments list thresholds in ADC counts and DC Volts.
  //
  //                                           NOM   ----MIN----  -----MAX----
  if (adcCounts > 882)  return KEY_NONE;   //  1023  882 (4.32V)  1023 (5.00V) 
  if (adcCounts > 623)  return KEY_SELECT; //   741  624 (3.05V)   882 (4.31V) 
  if (adcCounts > 417)  return KEY_LEFT;   //   505  418 (2.04V)   623 (3.04V) 
  if (adcCounts > 237)  return KEY_DOWN;   //   329  238 (1.16V)   417 (2.04V)
  if (adcCounts >  72)  return KEY_UP;     //   145   73 (0.36V)   237 (1.16V) 
  /* default */         return KEY_RIGHT;  //     0      n/a        72 (0.35V)
}





