/*      PWM Motor Speed Controller Data Types - SpdCtrlTypes.h
 
 Header file defining the enumerated data types used in the
 PWM Motor Speed Controller sketch.  
 
 The compiler must know what a function looks like before it can be used.
 This is done by a "prototype" that defines the function without its body.
 These prototypes are placed in "header" files and read first, so all is
 known when building the sketch.  The Aduino IDE automatically creates
 function prototypes and inserts them AFTER the headers, but BEFORE any
 other code.  Therefore, if any of our functions use a custom data type,
 they MUST be defined in a header file like this one or we will get a
 compiler error at line -1: 'MyType' does not name a type
 
 Vers   ---Date----   --Author--   Description-------------------------
 1.00   12-Jun-2012    R.Waters    Created to hold enumerated datatypes

*************************************************************************/

#ifndef SPDCTLTYPES_h // make sure we only process this header ONCE
#define SPDCTLTYPES_h 
//
//   @@@@@
//   @
//   @     @@@@  @   @  @@@ @@   @@@@
//   @@@@@ @   @ @   @  @  @  @ @
//   @     @   @ @   @  @  @  @  @@@
//   @     @   @ @   @  @  @  @     @
//   @@@@@ @   @  @@@@  @  @  @ @@@@
//
// An enumerated data type defines a set of named values that are implemented
// internally as integers.  Default assigned values are 1, 2, 3, ...
// This program often handles enum values as integers because the compiler
// will not accept the enum type name in a function argument list or for
// a function's return type.
//_____________________________________________________________________________
//
// Define the user interface modes AND the sequence in which they
// are selected by the up/down keys on the LCD/keypad shield.
// If you modify the list below, you may need to update the function
// eSelectUIMode() so that mode sequencing and wraparound will work.
//
typedef enum UI_MODE
{
  MODE_RUN_DEFAULT,       // Show motor cmd, RPM, surf speed
  
  MODE_SET_TOOL_DIAM,     // Set "Tool Diameter" in 1/16" (SAE) or 1 mm (SI) steps
  MODE_SET_MAX_POWER,     // Set "Max Motor Power" in range 5% to 100%
  MODE_SET_ACCEL_RATE,    // Set "Motor Accel Rate"  (Decel Rate is TBD) as %/sec
  
  MODE_SET_TACH_PPR,      // Set "RPM Sensor" as "disabled" or "x pulses/rev"
  MODE_SET_TACH_CALIB,    // Set the tachometer calibration factor
  MODE_SET_TACH_SCALE,    // Set the tachometer mechanical scale factor
  MODE_SET_UOM,           // Set Units of Measure as SAE (16th in) or SI (mm)
  
  MODE_SET_SW_INPUT,      // Set source of START/STOP commands
  MODE_SET_SW_THRESHOLD_U,// Set switch voltage for RUN
  MODE_SET_SW_THRESHOLD_L,// Set switch voltage for STOP
  
  MODE_SET_POT_INPUT,     // Set source of analog speed control command
  MODE_SET_POT_LIMIT_U,   // Set pot voltage commanding full power
  MODE_SET_POT_LIMIT_L,   // Set pot voltage commanding zero power
  
  MODE_SET_AUX_KEYPAD,    // Set Auxiliary Keypad on ADC5
  
  MODE_SET_DEFAULTS,      // Set "factory" default values into EEPROM
  MODE_SHOW_DIAG,         // Show selected ADC input; reset to exit?
  MODE_SHOW_VERSION,      // Show firmware version number
  MODE_SET_BACKLIGHT      // Set "LCD Backlight" as Off -or- xxx%
};

//_____________________________________________________________________________
//
// Define enumerated (integer) keypad ID codes.  The first 5 are defined for
// the Cytron or DFRobot LCD/Keypad "shield" (option card) for the Arduino UNO.
// These commands are passed into the function handlers for each UI_MODE.
// 5 keys are read as discrete voltage levels input to ADC channel 0.
// Refer to the Read5KeyPad() routine below for more details on this.
// Note: The 6th key (on the right end of the pad) actuates CPU reset.
//
typedef enum KEY_ID
{
  KEY_NONE=0, // 0 = no key or button pressed
  
  // Keys on the Cytron or DFRobot LCD/Keypad "shield"
  KEY_RIGHT,  // 1
  KEY_UP,     // 2
  KEY_DOWN,   // 3
  KEY_LEFT,   // 4
  KEY_SELECT, // 5 = function varies with current UI_MODE
  VKEY_INIT,  // 6 = virtual key command to initialize a function
  
  // Contact closures (Button/Sw) on panel or floor
  BTN_STOP,   // 7 = Stop command from foot switch or pot
  BTN_RUN     // 8 = Start/Run command from foot switch or pot
};

#endif // #ifndef SPDCTLTYPES_h


